# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA, or go to
# http://www.gnu.org/copyleft/lesser.txt.
#-----------------------------------------------------------------------------
# Name		: su2kt.rb
# Description	: Export model,su2kt_lights & cameras for current view or for all SU pages into Kerkythea
# Menu Item	: Plugins\kerkythea Exporter
# Author	: Tomasz Marek
#		  Code used form following SU exporters: SU2POV by Didier Bur and OGRE exporter by Kojack
# Usage		: Copy script to PLUGINS folder in SketchUp folder, run SU, go to Plugins\Kerkythea exporter
# Date		: 2008-02-5
# Type		: Exporter
# Version	: 3.05 - Mac compatibile version
#
#			3.0 - Memory leak bug fixed, wrong date bug fixed (KT2007),intensity of colours back to 1.0,
#			KT icons added, model opening in KT after export, missing GIF textures bug fixed,
#			KT library materials import added, backface material exported when no front material specified
#			Time bug for non GMT location fixed, photomatched faces\textures exported as a selection,
#			Tool for placing spot & pointlights added
#		  2.21 - Flipped images and missing BMP textures bugs fixed
#		  2.2 - Bugfixes, automatic activation of lights,animated lights status added,
#			FakeEmit, Emit[power] material added, frame numbers inproved,
#			Animation settings stored with model, Rendering settings from main XML are preserved
#			SU2KT is now Mac enabled (Thanks to povlhp)
#		  2.11 - SU 6 fixes by Wehby
#		  2.1  - Animation export added, Face Me Components Export for current view
#		  2.01 - Selection, Clay export added, Currnet View exported
#		  2.0  - All geometry and textures exported and nested lights supported
#		  1.1  - sun and phisical sky added, parallel views exported
#		  1.01 - bugfix - script is exporting all perspectives, won't stop on parallel views
#-----------------------------------------------------------------------------

require 'sketchup.rb'

class SU2KT

# ---- Settings and global variables ----- #

def SU2KT::reset_global_variables

	$n_pointlights=0
	$n_spotlights=0
	$n_cameras=0
	$face=0
	$g_su2kt_scale = 0.0254
	$g_su2kt_copy_textures = true
	$g_su2kt_export_materials = true
	$g_su2kt_export_meshes = true
	$g_su2kt_export_lights = true
	$g_su2kt_model_name=""
	$g_su2kt_textures_prefix = "TX_"
	$g_su2kt_texturewriter=Sketchup.create_texture_writer
	$g_su2kt_model_textures={}
	$g_su2kt_count_tri = 0
	$g_su2kt_count_faces = 0
	$g_su2kt_lights = []
	$g_su2kt_materials = {}
	$g_su2kt_fm_materials = {}
	$g_su2kt_selected=false
	$exp_distorted = false
	$exp_default_uvs = false
	$g_su2kt_clay=false
	$animation=false
	$frame=0
	$parent_mat=[]
	$fm_comp=[]
	# PHP added code for determining OS. See if PATH starts with / 
	# USE SEPARATOR instead
	if (ENV['PATH'][0] == '/'[0])
		$g_su2kt_ds = "/" # directory separator for OS X
	else
		$g_su2kt_ds = "\\" # directory separator for Windows,
	end
end

##### ------  Export ------ ######

def SU2KT::export

	SU2KT.reset_global_variables

	$g_su2kt_selected=true if Sketchup.active_model.selection.length > 0
	continue=SU2KT.export_options_window
	return if continue==false
	SU2KT.file_export_window
	return if $g_su2kt_model_name==""

	model = Sketchup.active_model
	out = File.new($g_su2kt_export_file,"w")
	$g_su2kt_path_textures=File.dirname($g_su2kt_export_file)

	start_time=Time.new
	SU2KT.export_global_settings(out)

	entity_list=model.entities
	entity_list=model.selection if $g_su2kt_selected==true

	SU2KT.find_lights(entity_list,Geom::Transformation.new)

	SU2KT.write_sky(out)
	SU2KT.export_meshes(out)
	SU2KT.param_cam_view(model.active_view, out)
	$n_cameras=1

	model.pages.each do |p|
		SU2KT.param_cam_page(p, out)
	end

	SU2KT.export_lights(out) if $g_su2kt_export_lights==true
	SU2KT.write_sun(out)
	SU2KT.finish_close(out)
	stext=SU2KT.write_textures

	result=SU2KT.report_window(start_time,stext)
	SU2KT.kt_start if result==6

	SU2KT.reset_global_variables

end

def SU2KT::export_options_window

	export_selection = %w[Yes No].join("|")
	export_meshes = %w[Yes No].join("|")
	export_lights = %w[Yes No].join("|")
	export_clay = %w[Yes No].join("|")
	export_distorted= %w[Yes No].join("|")
	export_default_uvs= %w[Yes No].join("|")

	si = Sketchup.active_model.shadow_info
	tzoffset = si["TZOffset"].to_i
	tzoffset = 0 if tzoffset.integer? and tzoffset.abs>12

	dropdowns=[]
	values=[]
	prompts=[]
	if $g_su2kt_selected==true
		dropdowns.push export_selection
		prompts.push "Export selection ONLY   "
		values.push "Yes"
		add_item=1
	else
		add_item=0
	end

	dropdowns+=[export_meshes,export_lights,export_clay,export_distorted,export_default_uvs]
	prompts+=["Geometry","Lights","Clay","Photomatched                       ","Default UVs"]
	values+=["Yes","Yes","No","No","No"]
	results = inputbox prompts,values, dropdowns, "Export options"
	return false if not results
	if (add_item==1 &&  results[0]=="Yes")
		$g_su2kt_selected=true
	else
		$g_su2kt_selected=false
	end
	$g_su2kt_export_meshes = false if results[0+add_item]=="No"
	$g_su2kt_export_lights = false if results[1+add_item]=="No"
	($g_su2kt_clay=true;$g_su2kt_copy_textures = false) if results[2+add_item]=="Yes"
	$exp_distorted = true if results[3+add_item]=="Yes"
	$exp_default_uvs = true if results[4+add_item]=="Yes"
	true
end

def SU2KT::file_export_window

	export_text="Export Model to Kerkythea"
	export_text="Export SELECTION to Kerkythea" if $g_su2kt_selected==true

	model = Sketchup.active_model

	model_filename = File.basename(model.path)
		if model_filename!=""
			model_name = model_filename.split(".")[0]
			model_name += ".xml"
		else
			model_name = "Untitled.xml"
		end

		$g_su2kt_export_file=UI.savepanel(export_text, "" , model_name)

		return if $g_su2kt_export_file==nil

		if $g_su2kt_export_file==$g_su2kt_export_file.split(".")[0]
			$g_su2kt_export_file+=".xml"
		end
		$g_su2kt_model_name=File.basename($g_su2kt_export_file)
		$g_su2kt_model_name=$g_su2kt_model_name.split(".")[0]

end

def SU2KT::report_window(start_time,stext)

	end_time=Time.new
	elapsed=end_time-start_time
	time=" exported in "
		(time=time+"#{(elapsed/3600).floor}h ";elapsed-=(elapsed/3600).floor*3600) if (elapsed/3600).floor>0
		(time=time+"#{(elapsed/60).floor}m ";elapsed-=(elapsed/60).floor*60) if (elapsed/60).floor>0
		time=time+"#{elapsed.round}s. "

	SU2KT.status_bar(stext+time+" Triangles = #{$g_su2kt_count_tri}")
	export_text="Model & Lights saved in file:\n"
	export_text="Selection saved in file:\n" if $g_su2kt_selected==true
	result=UI.messagebox export_text + $g_su2kt_export_file + "\n\nCameras exported: " + $n_cameras.to_s + "\n" + $sunexport + "Lights exported:\nPointlights: " + $n_pointlights.to_s +  "   Spotlights: " + $n_spotlights.to_s+"\n\nOpen exported model in Kerkythea?",MB_YESNO

end

def SU2KT::render_animation

	kt_path=SU2KT.get_kt_path
	return if kt_path==nil
	script_file_path=$g_su2kt_export_file
	script_file_path=File.join(script_file_path.split($g_su2kt_ds))
	Thread.new do
		system(`#{kt_path} "#{script_file_path}"`)
	end
end

def SU2KT::kt_start

	kt_path=SU2KT.get_kt_path
	return if kt_path==nil

	script_file_path=File.dirname(kt_path)+$g_su2kt_ds+"start.kst"
	script_file_path=File.join(script_file_path.split($g_su2kt_ds))
	script=File.new(script_file_path,"w")
	script.puts "message \"Load #{$g_su2kt_export_file}\""
	SU2KT.generate_sun(script)
	script.close
	Thread.new do
		system(`#{kt_path} "#{script_file_path}"`)
	end

end

def SU2KT::get_kt_path

	path=File.dirname(__FILE__)+$g_su2kt_ds+"kt_path.txt"
	find_kt=false

	if File.exist?(path)  #check if kt_path.txt exists
		path_file=File.new(path,"r")
		kt_path=path_file.read  #contents of kt_path.txt
		path_file.close
		if SU2KT.kt_path_valid?(kt_path)
			return kt_path
		else
			find_kt=true
		end
	else
		find_kt=true
	end

	if find_kt==true
		kt_path=UI.openpanel("LOCATE Kerkythea Executable, PLEASE","","")
		return nil if kt_path==nil
		path_file=File.new(path,"w") if kt_path
		path_file.write(kt_path) if kt_path
		path_file.close if kt_path
	end

	if SU2KT.kt_path_valid?(kt_path)
		return kt_path
	else
		return nil
	end

end

def SU2KT::kt_path_valid?(kt_path)
	(File.exist?(kt_path) and (File.basename(kt_path).upcase.include?("KERKYTHEA"))) #check if the path to kt is valid
end

# ---------------- About message ------------- #
def SU2KT::about
UI.messagebox("SU2KT version 3.05 5th February 2008
Freeware SketchUp Exporter to Kerkythea
Author: Tomasz MAREK

USAGE:
1. Any component called 'su2kt_pointlight' or 'su2kt_spotlight'
    will turn into the corresponding light type. To edit it
    please select one and use context menu.
2. To export model to Kerkythea
   - go to Plugins\\Kerkythea Exporter and select 'Export model'
3. To assign Thin Glass shader to material
   add 'TG_' prefix to material's name.
4. To make material appear always bright call it 'EmitFake'
5. To turn material into light call it 'Emit[#]'
    where \# stands for power of light (8=~100W)
7. KT Material Libaries can be used now directy in SU.
    Please use KT Material Import tools.

For further information please visit
Kerkythea Website & Forum - www.kerkythea.net

Plug-in is based on Su2Pov written by Didier Bur
and OGRE exporter by Kojack" , MB_MULTILINE , "SU2KT - Model Exporter to Kerkythea")


end

# -----------Extract the camera parameters of the current view ------------ ###
def SU2KT::param_cam_view( v , out)

h = v.vpheight
w = v.vpwidth
user_camera = v.camera
user_pers = user_camera.perspective?
user_eye = user_camera.eye

user_x = user_camera.xaxis
user_y = user_camera.yaxis
user_z = user_camera.zaxis

	if( user_pers == true )
	user_foc = user_camera.focal_length
	else
		user_foc = 36
	end

	w_x_h = w.to_s + "x" + h.to_s
	w_x_h=$resolution if $animation==true
	SU2KT.write_camera("\#\# Current View \#\#", user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)

end

# ----------------------------------------Extract the camera parameters of a particular page
def SU2KT::param_cam_page( p , out)

return if p.label[0]==40 and p.label[-1]==41  # page label with brackets

	h = Sketchup.active_model.active_view.vpheight
	w = Sketchup.active_model.active_view.vpwidth
	user_camera = p.camera
	user_pers = user_camera.perspective?
	user_eye = user_camera.eye
	user_x = user_camera.xaxis
	user_y = user_camera.yaxis
	user_z = user_camera.zaxis

if( user_pers == true )
	user_foc = user_camera.focal_length
else
	user_foc = 36
end

w_x_h = w.to_s + "x" + h.to_s

(SU2KT.write_camera p.name, user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)

end

#---------------- Save camera to file ------------------------#

def SU2KT::write_camera( p, user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)

out.puts "<Object Identifier=\"./Cameras/"+ p +"\" Label=\"Pinhole Camera\" Name=\""+ p +"\" Type=\"Camera\">"
out.puts  "<Parameter Name=\"Focal Length\" Type=\"Real\" Value=\"" + "%.4f" % (user_foc/36) + "\" />"
out.puts  "<Parameter Name=\"Resolution\" Type=\"String\" Value=\"#{w_x_h}\"/>"

cx=user_eye.x.to_m.to_f
cy=user_eye.y.to_m.to_f
cz=user_eye.z.to_m.to_f

rvx=user_x.x.to_cm.to_f
rvy=user_x.y.to_cm.to_f
rvz=user_x.z.to_cm.to_f

dvx=0.0-user_y.x.to_cm.to_f
dvy=0.0-user_y.y.to_cm.to_f
dvz=0.0-user_y.z.to_cm.to_f

tvx=user_z.x.to_cm.to_f
tvy=user_z.y.to_cm.to_f
tvz=user_z.z.to_cm.to_f

	out.puts "<Parameter Name=\"Frame\" Type=\"Transform\" Value=\"" + "%.4f" % (rvx) + " " + "%.4f" % (dvx) + " " + "%.4f" % (tvx) + " " + "%.4f" % (cx)
	out.puts "%.4f" % (rvy) + " " + "%.4f" % (dvy) + " " + "%.4f" % (tvy)+ " " + "%.4f" % (cy)
	out.puts "%.4f" % (rvz) + " " + "%.4f" % (dvz) + " " + "%.4f" % (tvz)+ " " + "%.4f" % (cz)
	out.puts "\"/>"
	out.puts "<Parameter Name=\"Focus Distance\" Type=\"Real\" Value=\"1\"/>"
	out.puts "</Object>"

$n_cameras +=1

end

#### --------- Lights export main routine --------- ########

def SU2KT::export_lights(out)

	SU2KT.status_bar("Exporting lights...")
	SU2KT.write_point_lights(out)
	SU2KT.write_spotlights(out)

end

### ---- Find light by name ---  #####

def SU2KT::find_lights(entity_list,trans)

	for e in entity_list

		if e.class == Sketchup::Group and e.layer.visible? and e.visible?
			SU2KT.find_lights(e.entities, trans*e.transformation)
		end

		if e.class == Sketchup::ComponentInstance and e.layer.visible? and e.visible?

			def_name=e.definition.name

			if (def_name.include? "su2" and def_name.include? "_spotlight") or (def_name.include? "su2" and def_name.include?"_pointlight")
				transold=trans
				trans=trans*e.transformation
				$g_su2kt_lights = $g_su2kt_lights + [[def_name,e,trans]]
				trans=transold
			else
				SU2KT.find_lights(e.definition.entities, trans*e.transformation)
			end

		end

	end
end

# ----------------------------------- Outputs omni-directionnal light sources (point_lights)
def SU2KT::write_point_lights(out)

	$g_su2kt_lights.each do |lights|

		if lights[0].include? "_pointlight"
		glob_trans=lights[2]
		e=lights[1]

		SU2KT.set_default_pointlight(e,nil)

			# get its color and power
			params = SU2KT.get_light_params (e)

			if (params[4].length>1 and $animation==true)
				state=SU2KT.check_animate_state(params[4])
				params[2]=state if state!="DEF"
			end
			# Light status is "On"
			params[2].upcase!

			if params[2] == "ON"

			trans=glob_trans.to_a #light location

			x = trans[12]
			y = trans[13]
			z = trans[14]

			ptx=glob_trans.xaxis
			pty=glob_trans.yaxis
			ptz=glob_trans.zaxis

			out.puts "<Object Identifier=\"./Lights/" + params[0] + "\" Label=\"Default Light\" Name=\"" + params[0] + "\" Type=\"Light\">"
			out.puts "<Object Identifier=\"Omni Light\" Label=\"Omni Light\" Name=\"\" Type=\"Emittance\">"
			out.puts "<Object Identifier=\"./Radiance/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
			out.puts "<Parameter Name=\"Color\" Type=\"RGB\" Value=\"" + params[3] + "\" />"
			out.puts "</Object>"
			out.puts "<Parameter Name=\"Attenuation\" Type=\"String\" Value=\"Inverse Square\"/>"
			out.puts "</Object>"
			out.puts "<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Shadow\" Type=\"Boolean\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Soft Shadow\" Type=\"Boolean\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Negative Light\" Type=\"Boolean\" Value=\"0\"/>"
			out.puts "<Parameter Name=\"Global Photons\" Type=\"Boolean\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Caustic Photons\" Type=\"Boolean\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Multiplier\" Type=\"Real\" Value=\"" + params[1].to_s + "\"/>"
			out.puts "<Parameter Name=\"Frame\" Type=\"Transform\" Value=\"" + "%.4f" % (ptx.x) + " " + "%.4f" % (pty.x) + " " + "%.4f" % (0.0-ptz.x) + " " + "%.4f" % (x.to_m)
			out.puts "%.4f" % (ptx.y) + " " + "%.4f" % (pty.y) + " " + "%.4f" % (0.0-ptz.y) + " " + "%.4f" % (y.to_m)
			out.puts "%.4f" % (ptx.z) + " " + "%.4f" % (pty.z) + " " + "%.4f" % (0.0-ptz.z) + " " + "%.4f" % (z.to_m)
			out.puts "\"/>"
			out.puts "<Parameter Name=\"Focus Distance\" Type=\"Real\" Value=\"1\"/>"
			out.puts "<Parameter Name=\"Radius\" Type=\"Real\" Value=\"0\"/>"
			out.puts "<Parameter Name=\"Shadow Color\" Type=\"RGB\" Value=\"0 0 0\"/>"
			out.puts "</Object>"

			$n_pointlights +=1

		end

	end
end
end

# -------------Outputs spotlight sources ----------------------  ###
def SU2KT::write_spotlights(out)

params = []
rad_fall_tight = []

$g_su2kt_lights.each do |lights|

	if lights[0].include? "_spotlight"
		glob_trans=lights[2]
		e=lights[1]

		SU2KT.set_default_spotlight(e,nil)
		params = SU2KT.get_light_params (e)

		if (params[4].length>1 and $animation==true)
				state=SU2KT.check_animate_state(params[4])
				params[2]=state if state!="DEF"
		end

	params[2].upcase!

	if( params[2] == "ON" )	# Light status is "On"

	rad_fall_tight = (SU2KT.get_spotlight_rad_fall_tight e)

	trans = glob_trans.to_a

	x = trans[12]
	y = trans[13]
	z = trans[14]

	ptx=glob_trans.xaxis
	pty=glob_trans.yaxis
	ptz=glob_trans.zaxis


	out.puts "<Object Identifier=\"./Lights/" + params[0] + "\" Label=\"Default Light\" Name=\"" + params[0] + "\" Type=\"Light\">"
	out.puts "<Object Identifier=\"Spot Light\" Label=\"Spot Light\" Name=\"\" Type=\"Emittance\">"
	out.puts "<Object Identifier=\"./Radiance/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "<Parameter Name=\"Color\" Type=\"RGB\" Value=\"" + params[3] + "\" />"
	out.puts "</Object>"
	out.puts "<Parameter Name=\"Attenuation\" Type=\"String\" Value=\"Inverse Square\"/>"

	falloff = rad_fall_tight[1].to_s
	out.puts "<Parameter Name=\"Fall Off\" Type=\"Real\" Value=\"" + falloff + "\"/>"
	radius = rad_fall_tight[0].to_s
	out.puts "<Parameter Name=\"Hot Spot\" Type=\"Real\" Value=\"" + radius + "\"/>"

	out.puts "</Object>"
	out.puts "<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Shadow\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Soft Shadow\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Negative Light\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "<Parameter Name=\"Global Photons\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Caustic Photons\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Multiplier\" Type=\"Real\" Value=\"" + params[1].to_s + "\"/>"
	out.puts "<Parameter Name=\"Frame\" Type=\"Transform\" Value=\"" + "%.4f" % (ptx.x) + " " + "%.4f" % (pty.x) + " " + "%.4f" % (0.0-ptz.x) + " " + "%.4f" % (x.to_m)
	out.puts "%.4f" % (ptx.y) + " " + "%.4f" % (pty.y) + " " + "%.4f" % (0.0-ptz.y) + " " + "%.4f" % (y.to_m)
	out.puts "%.4f" % (ptx.z) + " " + "%.4f" % (pty.z) + " " + "%.4f" % (0.0-ptz.z) + " " + "%.4f" % (z.to_m)
	out.puts "\"/>"
	dist=e.get_attribute("su2kt", "distance")
	dist=1 if dist==nil
	out.puts "<Parameter Name=\"Focus Distance\" Type=\"Real\" Value=\"#{dist}\"/>"
	out.puts "<Parameter Name=\"Radius\" Type=\"Real\" Value=\"0\"/>"
	out.puts "<Parameter Name=\"Shadow Color\" Type=\"RGB\" Value=\"0 0 0\"/>"
	out.puts "</Object>"

	$n_spotlights +=1

	end

	end
end

end

# ----------------------------------- Get the radius, falloff and tightness parameters of a spotlight
def SU2KT::get_spotlight_rad_fall_tight( light )

dict_name="su2kt"
dicts=light.attribute_dictionaries
ret = []
if( dicts != nil and dicts[dict_name] )
	spotlight_rad = light.get_attribute(dict_name, "radius").to_f
	fall = light.get_attribute(dict_name, "falloff").to_f
	else
	ret = ["0", "100"]
end
# default values if not set by the user
if( spotlight_rad == nil )
	ret = ["0", "100"]
	else
	ret = [spotlight_rad, fall]
end
return ret
end

# ----------------------------------- Set the color and power of selected pointlight
def SU2KT::set_pointlight_params(e,dist)

	# Get the pointlight color, even if not already set by the user
	if e.material != nil
		pointlight_color = e.material.display_name
	else
		pointlight_color = "White"
	end

	# Retrieve color and power if any
	SU2KT.set_default_pointlight(e,dist)

	dict_name="su2kt"
	dict_key_name = e.get_attribute(dict_name, "name")
	dict_key_power = e.get_attribute(dict_name, "power")
	dict_key_status = e.get_attribute(dict_name, "status")
	dict_key_animate = e.get_attribute(dict_name, "animate")

#Dialog box 1: Name, power
	status_list = %w[On Off].join("|")
	dropdowns = [status_list]
	prompts=["Status  ","Light name  ", "Light power  ","Animation keys ", "Sample: "]
	values=[dict_key_status, dict_key_name, dict_key_power,dict_key_animate,"1[On]-4[Off]-7[On]                   "]
	results = inputbox prompts, values, dropdowns, "LIGHT COLOUR: " + pointlight_color
	return nil if not results
	dict_key_name = results[1]
	dict_key_status = results[0]
	dict_key_power = results[2]
	dict_key_animate = results[3]

	e.attribute_dictionary(dict_name, true)
	e.set_attribute(dict_name,"name",dict_key_name)
	e.set_attribute(dict_name,"power",dict_key_power)
	e.set_attribute(dict_name,"status",dict_key_status)
	e.set_attribute(dict_name,"animate",dict_key_animate)
end

# ----------------------------------- Set the parameters of selected spotlight
def SU2KT::set_spotlight_params(e,dist)

	SU2KT.set_default_spotlight(e,dist)
	dict_name="su2kt"
	dict_key_name = e.get_attribute(dict_name, "name")
	dict_key_power = e.get_attribute(dict_name, "power")
	dict_key_status = e.get_attribute(dict_name, "status")
	dict_key_radius = e.get_attribute(dict_name, "radius")
	dict_key_falloff = e.get_attribute(dict_name, "falloff")
	dict_key_animate = e.get_attribute(dict_name, "animate")
	dict_key_radius = $su2kt_spothot if $su2kt_spothot!=nil
	dict_key_falloff = $su2kt_spotoff if $su2kt_spotoff!=nil
# Get color of spotlight, even if not set by the user
	if e.material != nil
	spotlight_color = e.material.display_name
	else
	spotlight_color = "White"
	end
#Dialog box 1:
	status_list = %w[On Off].join("|")
	dropdowns = [status_list]
	prompts=["Status  ","Light name  ","Light power ", "Hot Spot ", "Falloff  ", "Animation keys  ","Sample: "]
	values=[dict_key_status,dict_key_name, dict_key_power, dict_key_radius, dict_key_falloff,dict_key_animate,"1[On]-4[Off]-7[On]                   "]
	results = inputbox prompts,values, dropdowns, "Spotlight Color: " + spotlight_color
	return nil if not results
	dict_key_name = results[1]
	dict_key_power = results[2]
	dict_key_radius = results[3]
	dict_key_falloff = results[4]
	dict_key_status = results[0]
	dict_key_animate=results[5]

	dict_name="su2kt"
	e.attribute_dictionary(dict_name, true)
	e.set_attribute(dict_name,"name",dict_key_name)
	e.set_attribute(dict_name,"power",dict_key_power)
	e.set_attribute(dict_name,"status",dict_key_status)
	e.set_attribute(dict_name,"radius",dict_key_radius)
	e.set_attribute(dict_name,"falloff",dict_key_falloff)
	e.set_attribute(dict_name,"animate",dict_key_animate)
	$su2kt_spothot = dict_key_radius
	$su2kt_spotoff = dict_key_falloff

end

def SU2KT::get_object_color (obj)
	if (obj.material == nil)
		col = Sketchup::Color.new [255,255,255]
	else
		col = obj.material.color
	end
return col
end

# ---- Get the params of light ----- #
# ---- returns an array -------------#
def SU2KT::get_light_params( light )

	dict_name="su2kt"
	dict=light.attribute_dictionary "su2kt"

	if dict != nil
		light_name = light.get_attribute(dict_name, "name")
		pow = light.get_attribute(dict_name, "power").to_f
		status = light.get_attribute(dict_name, "status")
		animate = light.get_attribute(dict_name, "animate")
	else
		ret = ["noname","1","On","1.0 1.0 1.0"," "]
	end

	# get color of light
		col = (SU2KT.get_object_color light)
	# Component found, real values or default values if not set by the user
		if( col != nil)
			red = (col.red / 255.0)
			green = (col.green / 255.0)
			blue = (col.blue / 255.0)
			return_col = red.to_s + " " + green.to_s + " " + blue.to_s
			ret = [light_name, pow, status, return_col,animate]
		else
			ret = ["noname","1","On","1.0 1.0 1.0"," "]
		end
	return ret
end

### ---- set default pointlight values if not present --- ####

def SU2KT::set_default_pointlight(e,dist)

	dict_name="su2kt"

	if !e.attribute_dictionary dict_name
		if e.attribute_dictionary "su2pov"
			dict_key_name = e.get_attribute("su2pov", "name")
			dict_key_power = e.get_attribute("su2pov", "power")
			dict_key_status = e.get_attribute("su2pov", "status")
		else #default values
		dict_key_name = "Pointlight"
		dict_key_power = "%.1f" % (0.7*dist*dist) if dist
		dict_key_status = "On"
		end
	dict_key_animate = " "
	e.attribute_dictionary(dict_name, true)
	e.set_attribute(dict_name,"name",dict_key_name)
	e.set_attribute(dict_name,"power",dict_key_power)
	e.set_attribute(dict_name,"status",dict_key_status)
	e.set_attribute(dict_name,"animate",dict_key_animate)
	end

end

### ---- set default spotlight values if not present --- ####

def SU2KT::set_default_spotlight(e,dist)

	dict_name="su2kt"

	if !e.attribute_dictionary dict_name
		if e.attribute_dictionary "su2pov"
			dict_key_name = e.get_attribute("su2pov", "name")
			dict_key_power = e.get_attribute("su2pov", "power")
			dict_key_status = e.get_attribute("su2pov", "status")
			dict_key_radius = e.get_attribute("su2pov", "radius")
			dict_key_falloff = e.get_attribute("su2pov", "falloff")
		else #default values
		dict_key_name = "Spotlight"
		dict_key_power = "5"
		dict_key_power = "%.1f" % (1*dist*dist) if dist
		dict_key_status = "On"
		dict_key_radius = "0"
		dict_key_falloff = "100"
		dict_key_dist = dist
		end
	dict_key_animate = " "
	e.attribute_dictionary(dict_name, true)
	e.set_attribute(dict_name,"name",dict_key_name)
	e.set_attribute(dict_name,"power",dict_key_power)
	e.set_attribute(dict_name,"status",dict_key_status)
	e.set_attribute(dict_name,"radius",dict_key_radius)
	e.set_attribute(dict_name,"falloff",dict_key_falloff)
	e.set_attribute(dict_name,"animate",dict_key_animate)
	e.set_attribute(dict_name,"distance",dict_key_dist)
	end

end

# ---- Check light state & intensity in specified animation frame ---- ###

def SU2KT::check_animate_state(animate)

	steps=animate.split('-')
	steps.length.times do |i|
		steps[i]=steps[i].split("[")
		steps[i][1].delete!"]" if steps[i][1]
	end

	timer=$frame/$frame_per_sec.to_f
	i=-1
	while (steps.length-1)>i and timer>steps[i][0].to_f
		timer-=steps[i][0].to_f
		i=i+1
	end

	if $frame/$frame_per_sec.to_f>=steps[0][0].to_f
		ret=steps[i][1]
	else
		ret="DEF"
	end

return ret
end

# -------------------------------- Export Phisical Sky and other settings

def SU2KT::write_sky ( out )
si = Sketchup.active_model.shadow_info
if SU2KT.enable_sun
background="Physical Sky"
else
background="Background Color"
end

	si = Sketchup.active_model.shadow_info
	v = si["SunDirection"]

	point = Geom::Point3d.new 0,0,0
	xvector = Geom::Vector3d.new 1,0,0
	yvector = Geom::Vector3d.new 0,1,0
	zvector = Geom::Vector3d.new 0,0,1
	nangle = si["NorthAngle"].degrees
	tr = Geom::Transformation.rotation point, zvector, -nangle
	tr2 = Geom::Transformation.rotation point, zvector, nangle
	ptx=xvector.transform! tr
	pty=yvector.transform! tr
	v=v.transform! tr2		#Sun direction has to be rotated in opposite direction..

	result=SU2KT.geo_location(false)
	lati=result[0]
	longi=result[1]
	tzoffset=result[2]
	date=result[3]
	time=result[4]

out.puts "<Object Identifier=\"Default Global Settings\" Label=\"Default Global Settings\" Name=\"\" Type=\"Global Settings\">"
out.puts "	<Parameter Name=\"Ambient Light\" Type=\"RGB\" Value=\"0 0 0\"/>"

out.puts "	<Parameter Name=\"Background Color\" Type=\"RGB\" Value=\"0 0 0\"/>"

out.puts "	<Parameter Name=\"Compute Volume Transfer\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"Transfer Recursion Depth\" Type=\"Integer\" Value=\"1\"/>"

out.puts "	<Parameter Name=\"Background Type\" Type=\"String\" Value=\"" + background + "\"/>"
out.puts "	<Parameter Name=\"Sky Intensity\" Type=\"Real\" Value=\"1\"/>"

out.puts "	<Parameter Name=\"Sky Frame\" Type=\"Transform\" Value=\""+"%.4f"%(ptx.x)+" "+"%.4f"%(pty.x)+" 0 0 "+"%.4f"%(ptx.y)+" "+ "%.4f" % (pty.y) + " 0 0 " + "%.4f" % (ptx.z) + " " + "%.4f" % (pty.z) + " 1 0\"/>"

out.puts "	<Parameter Name=\"Sun Direction\" Type=\"String\" Value=\""+ "%.4f" % (v.x) + " " + "%.4f" % (v.y) + " " + "%.4f" % (v.z) + "\"/>"
out.puts "	<Parameter Name=\"Sky Turbidity\" Type=\"Real\" Value=\"2\"/>"
out.puts "	<Parameter Name=\"./Location/Latitude\" Type=\"Real\" Value=\""+ "%.4f" % (lati) + "\"/>"
out.puts "	<Parameter Name=\"./Location/Longitude\" Type=\"Real\" Value=\"" + "%.4f" % (longi) + "\"/>"
out.puts "	<Parameter Name=\"./Location/Timezone\" Type=\"Integer\" Value=\"" + "%.0f" % (tzoffset) + "\"/>"
out.puts "	<Parameter Name=\"./Location/Date\" Type=\"String\" Value=\"#{date}\"/>"
out.puts "	<Parameter Name=\"./Location/Time\" Type=\"String\" Value=\"#{time}\"/>"
out.puts "</Object>"

end


# ----------------------------------- Export sun possition
def SU2KT::write_sun(out)

	$sunexport=""

	if SU2KT.enable_sun==true
		enabled="1"
	else
		enabled="0"
	end

	sun_power ="3.0" # KT won't calculate it when opening the file, it has to be harcoded
	sun_col = "1 1 1"

	si = Sketchup.active_model.shadow_info
	v = si["SunDirection"]

	boundbox=Sketchup.active_model.bounds
	wsp= boundbox.center
	factor = 50
	pos = wsp.offset(v,[boundbox.max.x,boundbox.max.y,boundbox.max.z].max * factor)

	x=pos.x
	y=pos.y
	z=pos.z

	out.puts "<Object Identifier=\"./Lights/Sun\" Label=\"Default Light\" Name=\"Sun\" Type=\"Light\">"
	out.puts "<Object Identifier=\"Omni Light\" Label=\"Omni Light\" Name=\"\" Type=\"Emittance\">"
	out.puts "<Object Identifier=\"./Radiance/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "<Parameter Name=\"Color\" Type=\"RGB\" Value=\"" + sun_col + "\" />"
	out.puts "</Object>"
	out.puts "<Parameter Name=\"Attenuation\" Type=\"String\" Value=\"None\"/>"
	out.puts "</Object>"
	out.puts "<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"" + enabled + "\"/>"
	out.puts "<Parameter Name=\"Shadow\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Soft Shadow\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "<Parameter Name=\"Negative Light\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "<Parameter Name=\"Global Photons\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Caustic Photons\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Multiplier\" Type=\"Real\" Value=\"" + sun_power.to_s + "\"/>"
	out.puts "<Parameter Name=\"Frame\" Type=\"Transform\" Value=\"1 0 0 #{x.to_m.to_s} 0 1 0 #{y.to_m.to_s} 0 0 1 #{z.to_m.to_s}\"/>"
	out.puts "<Parameter Name=\"Focus Distance\" Type=\"Real\" Value=\"4\"/>"
	out.puts "<Parameter Name=\"Radius\" Type=\"Real\" Value=\"0.2\"/>"
	out.puts "<Parameter Name=\"Shadow Color\" Type=\"RGB\" Value=\"0 0 0\"/>"
	out.puts "</Object>"

	if enabled=="1"
		enabled="ON"
	else
		enabled="OFF"
	end

	$sunexport="Sun & Phisical Sky exported and turned " + enabled +".           \n"
	$g_su2kt_lights = []
end

#### Sun related methods

def SU2KT::geo_location(script)

	si = Sketchup.active_model.shadow_info
	location = si["City"]
	country = si["Country"]

	lati= si["Latitude"]
	longi= si["Longitude"]

	tzoffset = si["TZOffset"]
	tzoffset = 0 if tzoffset.abs>12

	date = si["ShadowTime"].utc #Thanks Wehby.. I know, I know
	sec = date.sec
	min = date.min

	if date.isdst==true # DaySavingTime
		hour = date.hour-1
		else
		hour = date.hour
	end

	day = date.day
	month = date.month
	if script != true
		month-=1 #Zero indexed month in KT
		day-=1 #Zero indexed day in KT
	end
	time_now=Time.now
	year = time_now.year

	return [lati,longi,tzoffset,"#{day}-#{month}-#{year}","#{hour}:#{min}:#{sec}"] if script == true
	return [lati,longi,tzoffset,"#{day}/#{month}/#{year}","#{hour}:#{min}:#{sec}"] if script != true

end

def SU2KT::generate_sun(out)
	result=SU2KT.geo_location(true)
	lati=result[0]
	longi=result[1]
	tzoffset=result[2]
	date=result[3]
	time=result[4]

	out.puts "message \"./Scenes/#{$g_su2kt_model_name}/GenerateSun #{longi} #{lati} GT#{tzoffset.to_i} #{date} #{time}\""
end

def SU2KT::enable_sun
	$g_su2kt_lights==[] or ($g_su2kt_lights!=[] and Sketchup.active_model.shadow_info["DisplayShadows"])
end

#### -------------- Export rendering settings ----------------- ##########

def SU2KT::export_global_settings(out)

out.puts "<Root Label=\"Default Kernel\" Name=\"\" Type=\"Kernel\">"
out.puts "<Object Identifier=\"./Modellers/XML Modeller\" Label=\"XML Modeller\" Name=\"XML Modeller\" Type=\"Modeller\">"
out.puts "</Object>"

out.puts "<Object Identifier=\"./Ray Tracers/Standard Ray Tracer\" Label=\"Standard Ray Tracer\" Name=\"Standard Ray Tracer\" Type=\"Ray Tracer\">"
out.puts "	<Parameter Name=\"Rasterization\" Type=\"String\" Value=\"Auto\"/>"
out.puts "	<Parameter Name=\"Antialiasing\" Type=\"String\" Value=\"Production AA\"/>"
out.puts "	<Parameter Name=\"Antialiasing Filter\" Type=\"String\" Value=\"Mitchell-Netravali 0.5 0.8\"/>"
out.puts "	<Parameter Name=\"Antialiasing Threshold\" Type=\"Real\" Value=\"0.3\"/>"
out.puts "	<Parameter Name=\"Texture Filtering\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Brightness Threshold\" Type=\"Real\" Value=\"0.002\"/>"
out.puts "	<Parameter Name=\"Max Ray Tracing Depth\" Type=\"Integer\" Value=\"5\"/>"
out.puts "	<Parameter Name=\"Irradiance Scale\" Type=\"RGB\" Value=\"1 1 1\"/>"
out.puts "	<Parameter Name=\"Linear Lightflow\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Diffuse Samples\" Type=\"Integer\" Value=\"64\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Specular Samples\" Type=\"Integer\" Value=\"8\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Dispersion Samples\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Diffusers\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Translucencies\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Fuzzy Reflections\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Fuzzy Refractions\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Reflections\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Refractions\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Random Generator\" Type=\"String\" Value=\"Pure\"/>"
out.puts "</Object>"
out.puts "<Object Identifier=\"./Irradiance Estimators/Diffuse Interreflection\" Label=\"Diffuse Interreflection\" Name=\"Diffuse Interreflection\" Type=\"Irradiance Estimator\">"
out.puts "	<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Max Recursion Depth\" Type=\"Integer\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Max Ray Tracing Depth\" Type=\"Integer\" Value=\"5\"/>"
out.puts "	<Parameter Name=\"Final Gathering Rays\" Type=\"Integer\" Value=\"500\"/>"
out.puts "	<Parameter Name=\"Light Sampling\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Ambient Lighting\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"Accuracy\" Type=\"Real\" Value=\"0.25\"/>"
out.puts "	<Parameter Name=\"Minimum Pixel Reuse\" Type=\"Real\" Value=\"2.5\"/>"
out.puts "	<Parameter Name=\"Maximum Pixel Reuse\" Type=\"Real\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"Radiance Limit\" Type=\"Real\" Value=\"0.3\"/>"
out.puts "	<Parameter Name=\"Secondary Estimator\" Type=\"String\" Value=\"Density Estimation\"/>"
out.puts "</Object>"
out.puts "<Object Identifier=\"./Irradiance Estimators/Density Estimation\" Label=\"Density Estimation\" Name=\"Density Estimation\" Type=\"Irradiance Estimator\">"
out.puts "	<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Direct Lighting\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Max Photon Tracing Depth\" Type=\"Integer\" Value=\"6\"/>"
out.puts "	<Parameter Name=\"Terminating Brightness\" Type=\"Real\" Value=\"0.01\"/>"
out.puts "	<Parameter Name=\"Samples per Light\" Type=\"Integer\" Value=\"100000\"/>"
out.puts "	<Parameter Name=\"Sample Sky\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Diffuse Samples\" Type=\"Integer\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Specular Samples\" Type=\"Integer\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Dispersion Samples\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Reflections\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Trace Refractions\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Importance Sampling\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"./Sampling Criteria/Russian Roulette\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "</Object>"
out.puts "<Object Identifier=\"./Direct Light Estimators/Refraction Enhanced\" Label=\"Refraction Enhanced\" Name=\"Refraction Enhanced\" Type=\"Direct Light Estimator\">"
out.puts "	<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"PseudoCaustics\" Type=\"Boolean\" Value=\"0\"/>"
out.puts "	<Parameter Name=\"Antialiasing\" Type=\"String\" Value=\"Low\"/>"
out.puts "	<Parameter Name=\"Optimized Area Lights\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "</Object>"
out.puts "<Object Identifier=\"./Environments/Octree Environment\" Label=\"Octree Environment\" Name=\"Octree Environment\" Type=\"Environment\">"
out.puts "	<Parameter Name=\"Max Objects per Cell\" Type=\"Integer\" Value=\"20\"/>"
out.puts "</Object>"
out.puts "<Object Identifier=\"./Filters/Simple Tone Mapping\" Label=\"Simple Tone Mapping\" Name=\"\" Type=\"Filter\">"
out.puts "	<Parameter Name=\"Enabled\" Type=\"Boolean\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Method\" Type=\"String\" Value=\"Simple\"/>"
out.puts "	<Parameter Name=\"Exposure\" Type=\"Real\" Value=\"1\"/>"
out.puts "	<Parameter Name=\"Gamma Correction\" Type=\"Real\" Value=\"2.2\"/>"
out.puts "</Object>"

out.puts "<Object Identifier=\"./Scenes/#{$g_su2kt_model_name}\" Label=\"Default Scene\" Name=\"#{$g_su2kt_model_name}\" Type=\"Scene\">"

end

def SU2KT::finish_close(out)

	out.puts "<Parameter Name=\"./Cameras/Active\" Type=\"String\" Value=\"\#\# Current View \#\#\"/>"
	out.puts "</Object>" #Close scene
	out.puts "</Root>"
	out.close

end

#### -------- Mesh export ---------- ######

def SU2KT::export_meshes(export_name)

	$parent_mat=[]

	if $g_su2kt_export_meshes == true

		if $g_su2kt_selected==true
			ents = []
			Sketchup.active_model.selection.each{|e| ents = ents + [e]}
			SU2KT.collect_entities(ents, Geom::Transformation.new)
		else
			SU2KT.collect_entities(Sketchup.active_model.entities, Geom::Transformation.new)
		end

		SU2KT.export_faces(export_name)
		SU2KT.export_fm_faces(export_name)

	end
end

###########

def SU2KT::export_faces(export_name)
	$g_su2kt_materials.each{|mat,value|
		if (value!=nil and value!=[])
			SU2KT.export_face(export_name,mat,false)
			$g_su2kt_materials[mat]=nil
		end}
	$g_su2kt_materials={}

end

###########

def SU2KT::export_fm_faces(export_name)
	$g_su2kt_fm_materials.each{|mat,value|
		if (value!=nil and value!=[])
			SU2KT.export_face(export_name,mat,true)
			$g_su2kt_fm_materials[mat]=nil
		end}
	$g_su2kt_fm_materials={}
end

##### ------------ Collecting entities into an array -------------- ##########

def SU2KT::collect_entities(object, trans)

	if object.class == Sketchup::ComponentInstance
		entity_list=object.definition.entities
	elsif object.class == Sketchup::Group
		entity_list=object.entities
	else
		entity_list=object
	end

	text=""
	text="Component: " + object.definition.name if object.class == Sketchup::ComponentInstance
	text="Group" if object.class == Sketchup::Group

	SU2KT.status_bar("Collecting Faces - Level #{$parent_mat.size} - #{text}")

	for e in entity_list

		if (e.class == Sketchup::Group and e.layer.visible? and e.visible?)
			SU2KT.get_inside(e,trans,false) #e,trans,false - not FM component
		end

		if (e.class == Sketchup::ComponentInstance and e.layer.visible? and e.visible? and !e.definition.name.include? "_pointlight" and !e.definition.name.include? "_spotlight")
			SU2KT.get_inside(e,trans,e.definition.behavior.always_face_camera?) # e,trans, fm_component?
		end

		if (e.class == Sketchup::Face and e.layer.visible? and e.visible?)

		face_properties=SU2KT.find_face_material(e)
		mat=face_properties[0]
		uvHelp=face_properties[1]
		mat_dir=face_properties[2]

		if $fm_comp.last==true
			($g_su2kt_fm_materials[mat] ||= []) << [e,trans,uvHelp,mat_dir]
		else
			($g_su2kt_materials[mat] ||= []) << [e,trans,uvHelp,mat_dir] if ($animation==false)
		end
		$g_su2kt_count_faces+=1

		end

	end #for loop

end #method end

def SU2KT::get_inside(e,trans,face_me)
	$fm_comp.push(face_me)
		if e.material != nil
			mat = e.material
			$parent_mat.push(e.material)
			SU2KT.store_textured_entities(e,mat,true) if (mat.respond_to?(:texture) and mat.texture!=nil)
		else
			$parent_mat.push($parent_mat.last)
		end
		SU2KT.collect_entities(e, trans*e.transformation)
		$parent_mat.pop
	$fm_comp.pop
end

def SU2KT::find_face_material(e)
	mat="FrontFace"
	uvHelp=nil
	mat_dir=true
	if e.material!=nil
		mat=e.material
	else
		if e.back_material!=nil
			mat=e.back_material
			mat_dir=false
		else
			mat=$parent_mat.last if $parent_mat.last!=nil
		end
	end

	if (mat.respond_to?(:texture) and mat.texture !=nil)
		ret=SU2KT.store_textured_entities(e,mat,mat_dir)
		mat=ret[0]
		uvHelp=ret[1]
	end

	return [mat,uvHelp,mat_dir]
end


def SU2KT::store_textured_entities(e,mat,mat_dir)

	verb=false

	tw=$g_su2kt_texturewriter

	puts "MATERIAL: " + mat.display_name if verb==true
	uvHelp=nil
	number=0
	mat_name=mat.display_name.delete"<>[]"

	if (e.class==Sketchup::Group or e.class==Sketchup::ComponentInstance) and mat.respond_to?(:texture) and mat.texture!=nil
			txcount=tw.count
			handle=tw.load e
			tname=get_texture_name(mat_name,mat)
			$g_su2kt_model_textures[mat_name]=[0,e,mat_dir,handle,tname,mat] if (txcount!=tw.count and $g_su2kt_model_textures[mat_name]==nil)
			puts "GROUP #{mat_name} H:#{handle}\n#{$g_su2kt_model_textures[mat_name]}" if verb==true
	end

	if e.class==Sketchup::Face

		if  $exp_distorted==false
			handle = tw.load(e,mat_dir)
			tname=SU2KT.get_texture_name(mat_name,mat)
			$g_su2kt_model_textures[mat_name]=[0,e,mat_dir,handle,tname,mat] if $g_su2kt_model_textures[mat_name]==nil
			return [mat_name,uvHelp,mat_dir]
		else

			distorted=SU2KT.texture_distorted?(e,mat,mat_dir)

			txcount=tw.count
			handle = tw.load(e,mat_dir)
			tname=SU2KT.get_texture_name(mat_name,mat)

			if txcount!=tw.count #if new texture added to tw

				if $g_su2kt_model_textures[mat_name]==nil
					if distorted==true
						uvHelp=SU2KT.get_UVHelp(e,mat_dir)
						puts "FIRST DISTORTED FACE #{mat_name} #{handle} #{e}" if verb==true
					else
						unHelp=nil
						puts "FIRST FACE #{mat_name} #{handle} #{e}" if verb==true
					end
					$g_su2kt_model_textures[mat_name]=[0,e,mat_dir,handle,tname,mat]
				else
					ret=SU2KT.add_new_texture(mat_name,e,mat,handle,mat_dir)
					mat_name=ret[0]
					uvHelp=ret[1]
					puts "DISTORTED FACE #{mat_name} #{handle} #{e}" if verb==true
				end
			else
				$g_su2kt_model_textures.each{|key, value|
					if handle==value[3]
						mat_name=key
						uvHelp=SU2KT.get_UVHelp(e,mat_dir) if distorted==true
						puts "OLD MAT FACE #{key} #{handle} #{e} #{uvHelp}" if verb==true
					end}
			end
		end
	end
	puts "FINAL: #{[mat_name,uvHelp,mat_dir].to_s}" if verb==true
	return [mat_name,uvHelp,mat_dir]
end

def SU2KT::add_new_texture(mat_name,e,mat,handle,mat_dir)
	state=$g_su2kt_model_textures[mat_name]
	number=state[0]=state[0]+1
	mat_name=mat_name+number.to_s
	tname=SU2KT.get_texture_name(mat_name,mat)
	uvHelp=SU2KT.get_UVHelp(e,mat_dir)
	$g_su2kt_model_textures[mat_name]=[number,e,mat_dir,handle,tname,mat]
return [mat_name,uvHelp]
end

def SU2KT::get_texture_name(name,mat)
	ext=mat.texture.filename
	ext=ext[(ext.length-4)..ext.length]
	ext=".png" if (ext.upcase ==".BMP" or ext.upcase ==".GIF") #Texture writer converts BMP,GIF to PNG
	ext=".jpg" if ext.upcase[0]!=46
	s=name+ext
	s=$g_su2kt_textures_prefix+$g_su2kt_model_name+$g_su2kt_ds+s
end

def SU2KT::texture_distorted?(e,mat,mat_dir)

	distorted=false
	temp_tw=Sketchup.create_texture_writer
	model = Sketchup.active_model
	entities = model.active_entities
	model.start_operation "Group" #For Undo
	group=entities.add_group
	group.material = mat
	g_handle=temp_tw.load(group)
	temp_handle=temp_tw.load(e,mat_dir)
	entities.erase_entities group
	Sketchup.undo
	distorted=true if temp_handle!=g_handle
	temp_tw=nil
	return distorted

end

def SU2KT::get_UVHelp(e,mat_dir)
	if mat_dir==true
		front=true;back=false
	else
		front=false;back=true
	end
	uvHelp=nil
	tw=$g_su2kt_texturewriter
	#vertices=e.vertices
	#vertices.each {|v| puts v.position}
	uvHelp = e.get_UVHelper(front, back, tw)
end

def SU2KT::write_textures

	if ($g_su2kt_copy_textures == true and $g_su2kt_model_textures!={})

		if FileTest.exist? ($g_su2kt_path_textures+$g_su2kt_ds+$g_su2kt_textures_prefix+$g_su2kt_model_name)
		else
			Dir.mkdir($g_su2kt_path_textures+$g_su2kt_ds+$g_su2kt_textures_prefix+$g_su2kt_model_name)
		end

		tw=$g_su2kt_texturewriter
		number=$g_su2kt_model_textures.length
		count=1
		$g_su2kt_model_textures.each do |key, value|
			SU2KT.status_bar("Exporting texture "+count.to_s+"/"+number.to_s)
			if value[1].class== Sketchup::Face
			tw.write value[1], value[2], ($g_su2kt_path_textures+$g_su2kt_ds+value[4])
			else
			tw.write value[1], ($g_su2kt_path_textures+$g_su2kt_ds+value[4])
			end
			count+=1
		end

		status='ok' #TODO

		if status
		stext = "SU2KT: " + (count-1).to_s + " textures and model"
		else
			stext = "An error occured when exporting textures. Model"
		end
		$g_su2kt_texturewriter=nil
		$g_su2kt_model_textures=nil
	else
		stext = "Model"
	end

	return stext

end

###########

def SU2KT::point_to_vector(p)
	Geom::Vector3d.new(p.x,p.y,p.z)
end

##### ---- Exporting faces converted to polymeshes ---- #######

def SU2KT::export_face(out,mat,fm_mat)

	meshes = []
	polycount = 0
	pointcount = 0
	mirrored={}
	mat_dir={}
	default_mat={}
	distorted_uv={}
	if fm_mat
		export=$g_su2kt_fm_materials[mat]
	else
		export=$g_su2kt_materials[mat]
	end

	has_texture = false
	if mat.respond_to?(:name)
		matname = mat.display_name.delete"<>[]"
		has_texture = true if mat.texture!=nil
	else
		matname = mat
		has_texture=true if matname!="FrontFace"
	end

	matname="FM_"+matname if fm_mat

	for ft in export

		SU2KT.status_bar("Converting Faces to Meshes: " + matname + " #{$g_su2kt_count_faces}") if ($g_su2kt_count_faces/200.0==$g_su2kt_count_faces/200)

		polymesh = ft[0].mesh 5 if ft[3]==true
		polymesh = ft[0].mesh 6 if ft[3]==false
		trans=ft[1]
		default_mat[polymesh]=(ft[0].material==nil)
		distorted_uv[polymesh]=ft[2]
		mat_dir[polymesh]=ft[3]
		polymesh.transform! trans
		mirrored[polymesh] = true
		xa = SU2KT.point_to_vector(ft[1].xaxis)
		ya = SU2KT.point_to_vector(ft[1].yaxis)
		za = SU2KT.point_to_vector(ft[1].zaxis)
		xy = xa.cross(ya)
		xz = xa.cross(za)
		yz = ya.cross(za)
		if xy.dot(za) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if xz.dot(ya) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if yz.dot(xa) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end

		meshes = meshes + [polymesh]
		$g_su2kt_count_faces-=1
		polycount=polycount + polymesh.count_polygons
		pointcount=pointcount + polymesh.count_points
	end

	startindex = 0

# Exporting vertices

	SU2KT.status_bar("Material being exported: " + matname + " - Verticles")

out.puts "<Object Identifier=\"./Models/#{matname}\" Label=\"Default Model\" Name=\"#{matname}\" Type=\"Model\">"
out.puts "	<Object Identifier=\"Triangular Mesh\" Label=\"Triangular Mesh\" Name=\"\" Type=\"Surface\">"
out.puts "	<Parameter Name=\"Vertex List\" Type=\"Point3D List\" Value=\"#{pointcount}\">"

	for mesh in meshes
		for p in (1..mesh.count_points)
			pos = mesh.point_at(p)
			out.print "	<P xyz=\"#{"%.4f" %(pos.x*$g_su2kt_scale)} #{"%.4f" %(pos.y*$g_su2kt_scale)} #{"%.4f" %(pos.z*$g_su2kt_scale)}\"/>\n"
		end
	end
	out.puts "	</Parameter>"

# Exporting normals

	SU2KT.status_bar("Material being exported: " + matname + " - Normals")

out.puts "	<Parameter Name=\"Normal List\" Type=\"Point3D List\" Value=\"#{pointcount}\">"

	for mesh in meshes
		for p in (1..mesh.count_points)
			norm = mesh.normal_at(p)
			norm.reverse! if mat_dir[mesh]==false
				out.print "	<P xyz=\"#{"%.4f" %(norm.x)} #{"%.4f" %(norm.y)} #{"%.4f" %(norm.z)}\"/>\n"
		end
	end
out.print "	</Parameter>\n"

# Exporting faces

	SU2KT.status_bar("Material being exported: " + matname + " - Faces")

	out.print "	<Parameter Name=\"Index List\" Type=\"Triangle Index List\" Value=\"#{polycount}\">\n"

	for mesh in meshes
		for poly in mesh.polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+startindex
			v2 = (poly[1]>=0?poly[1]:-poly[1])+startindex
			v3 = (poly[2]>=0?poly[2]:-poly[2])+startindex

			if !mirrored[mesh]
				if mat_dir[mesh]==true
					out.print "	<F ijk=\"#{v1-1} #{v2-1} #{v3-1}\"/>\n"
				else
					out.print "	<F ijk=\"#{v1-1} #{v3-1} #{v2-1}\"/>\n"
				end
			else
				if mat_dir[mesh]==true
					out.print "	<F ijk=\"#{v2-1} #{v1-1} #{v3-1}\"/>\n"
				else
					out.print "	<F ijk=\"#{v2-1} #{v3-1} #{v1-1}\"/>\n"
				end
			end

			$g_su2kt_count_tri = $g_su2kt_count_tri + 1
		end
		startindex = startindex + mesh.count_points
	end

	out.puts "	</Parameter>"
	out.puts "	<Parameter Name=\"Smooth\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"AA Tolerance\" Type=\"Real\" Value=\"15\"/>"
	out.print "	</Object>\n" #Closing Triangular Mesh

# Exporting Material

	if $g_su2kt_model_textures[matname]!=nil
	main_mat = $g_su2kt_model_textures[matname][5]
	else
	main_mat = mat
	end
	kt_attr=main_mat.attribute_dictionary "su2kt" if main_mat.class!=String
	if kt_attr!=nil and $g_su2kt_clay==false
		out.puts kt_attr["kt_mat"]
		SU2KT.add_bump(out,mat) if main_mat.respond_to?(:texture) and main_mat.texture!=nil and !kt_attr["kt_mat"].include?("Parameter Name=\"Filename\"")
	else
		if (main_mat.respond_to?(:name) and main_mat.display_name[0..2]=="TG_" and main_mat.use_alpha? and $g_su2kt_clay==false)
			SU2KT.export_thin_glass(out,mat)
		else
			SU2KT.export_phong(out,mat,main_mat) if $g_su2kt_clay==false
			SU2KT.export_clay(out) if $g_su2kt_clay==true
		end
	end

	#Exporting UVs

	if (has_texture and $g_su2kt_clay==false) or $exp_default_uvs==true
		out.print "	<Parameter Name=\"Map Channel\" Type=\"Point2D List\" Value=\"#{pointcount}\">\n"

		SU2KT.status_bar("Material being exported: " + matname + " - UVs")

		for mesh in meshes
		dir=true
		dir=mat_dir[mesh] if $exp_default_uvs==false
			for p in (1..mesh.count_points)
				default_mat[mesh]=false if $exp_default_uvs==true
				if default_mat[mesh] and dir==true
					mat_texture=($g_su2kt_model_textures[matname][5]).texture
					texsize = Geom::Point3d.new(mat_texture.width, mat_texture.height, 1)
				else
					texsize = Geom::Point3d.new(1,1,1)
					texsize = Geom::Point3d.new(100,100,100) if $exp_default_uvs==true
				end
				if distorted_uv[mesh]!=nil
					uvHelp=distorted_uv[mesh]
					uv=uvHelp.get_front_UVQ(mesh.point_at(p)) if mat_dir[mesh]==true
					uv=uvHelp.get_back_UVQ(mesh.point_at(p)) if mat_dir[mesh]==false
				else
					uv = [mesh.uv_at(p,dir).x/texsize.x, mesh.uv_at(p,dir).y/texsize.y, mesh.uv_at(p,dir).z/texsize.z]
				end
					out.puts "	<P xy=\"#{"%.4f" %(uv.x)} #{"%.4f" %(-uv.y+1)}\"/>"
			end
		end
		out.puts "	</Parameter>"
	else
		out.puts "	<Parameter Name=\"Map Channel\" Type=\"Point2D List\" Value=\"0\">"
		out.puts "	</Parameter>"
	end

	if kt_attr!=nil
		out.puts kt_attr["kt_map"]
	else
	out.puts "	<Parameter Name=\"Frame\" Type=\"Transform\" Value=\"1 0 0 0 0 1 0 0 0 0 1 0\"/>"
	out.puts "	<Parameter Name=\"Visible\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"Shadow Caster\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"Shadow Receiver\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"Caustics Transmitter\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"Caustics Receiver\" Type=\"Boolean\" Value=\"1\"/>"
	end

	out.puts "</Object>" #Close mesh description
end

##### --- EXPORT THIN GLASS -----#####
def SU2KT::export_thin_glass(out,mat)

	out.puts "<Object Identifier=\"Thin Glass Material\" Label=\"Thin Glass Material\" Name=\"\" Type=\"Material\">"
	if $g_su2kt_model_textures[mat]==nil
		material_color=mat.color
		clr=SU2KT.rgb_to_hsb(material_color.red,material_color.green,material_color.blue)
		clr[1]=clr[1]*mat.alpha
		res=hsb_to_rgb(clr[0],clr[1],clr[2])
		out.puts "	<Object Identifier=\"./Reflectance/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
		out.puts "	<Parameter Name=\"Color\" Type=\"RGB\" Value=\"#{res[0]/255.0} #{res[1]/255.0} #{res[2]/255.0}\"/>"
	else
		out.puts "	<Object Identifier=\"./Reflectance/Bitmap Texture\" Label=\"Bitmap Texture\" Name=\"\" Type=\"Texture\">"
		out.puts "	<Parameter Name=\"Filename\" Type=\"String\" Value=\"#{$g_su2kt_model_textures[mat][4]}\"/>"
		projection=SU2KT.override_projection
		out.puts projection
		out.puts "	<Parameter Name=\"Smooth\" Type=\"Boolean\" Value=\"1\"/>"
		out.puts "	<Parameter Name=\"Inverted\" Type=\"Boolean\" Value=\"0\"/>"
	end
	out.puts "	</Object>"
	out.puts "	<Parameter Name=\"Index of Refraction\" Type=\"Real\" Value=\"1.52\"/>"
	out.puts "</Object>"
end

##### --- EXPORT PHONG -----#####
def SU2KT::export_phong(out,mat,main_mat)

	lum_reduction = 1

	out.puts "<Object Identifier=\"Whitted Material\" Label=\"Whitted Material\" Name=\"\" Type=\"Material\">"

	if $g_su2kt_model_textures[mat]!=nil
	#TEXTURE
		out.puts "	<Object Identifier=\"./Diffuse/Weighted Texture\" Label=\"Weighted Texture\" Name=\"\" Type=\"Texture\">"
		out.puts "		<Object Identifier=\"Bitmap Texture\" Label=\"Bitmap Texture\" Name=\"\" Type=\"Texture\">"
		out.puts "		<Parameter Name=\"Filename\" Type=\"String\" Value=\"#{$g_su2kt_model_textures[mat][4]}\"/>"
		out.puts "		<Parameter Name=\"Projection\" Type=\"String\" Value=\"UV\"/>"
		out.puts "		</Object>"
		out.puts "	<Parameter Name=\"Bitmap Texture:Weight\" Type=\"Real\" Value=\"#{lum_reduction}\"/>"
		out.puts "</Object>"
	else
	#OR JUST A COLOR
		out.puts "	<Object Identifier=\"./Diffuse/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
		if main_mat.respond_to?(:name) and main_mat.texture==nil
			clr=SU2KT.rgb_to_hsb(mat.color.red,mat.color.green,mat.color.blue)
			clr[2]=clr[2]*lum_reduction
			result=hsb_to_rgb(clr[0],clr[1],clr[2])
			result=[216,216,216] if $g_su2kt_clay==true
			result=[255,255,255] if main_mat.display_name[0..7].upcase=="EMITFAKE"
			out.print "		<Parameter Name=\"Color\" Type=\"RGB\" Value=\"#{result[0]/255.0} #{result[1]/255.0} #{result[2]/255.0}\"/>\n"

		else
			colour = Sketchup.active_model.rendering_options["FaceFrontColor"]
			clr=SU2KT.rgb_to_hsb(colour.red,colour.green,colour.blue)
			clr[2]=clr[2]*lum_reduction
			result=hsb_to_rgb(clr[0],clr[1],clr[2])
			result=[216,216,216] if $g_su2kt_clay==true
			out.print "		<Parameter Name=\"Color\" Type=\"RGB\" Value=\"#{result[0]/255.0} #{result[1]/255.0} #{result[2]/255.0}\"/>\n"

		end
		out.puts "</Object>"
	end

	if (mat.respond_to?(:name) and mat.use_alpha?)
	ior=1
		out.puts "<Object Identifier=\"./Refraction/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
		out.puts "	<Parameter Name=\"Color\" Type=\"RGB\" Value=\"#{1.0-mat.alpha} #{1.0-mat.alpha} #{1.0-mat.alpha}\"/>"
		out.puts "</Object>"
	else
	ior=3
	end

	if $g_su2kt_model_textures[mat]!=nil
		out.print "	<Parameter Name=\"Smooth\" Type=\"Boolean\" Value=\"1\"/>\n"
		out.print "	<Parameter Name=\"Inverted\" Type=\"Boolean\" Value=\"0\"/>\n"
	end

	out.print "	<Parameter Name=\"Shininess\" Type=\"Real\" Value=\"128\"/>\n"
	out.print "	<Parameter Name=\"Transmitted Shininess\" Type=\"Real\" Value=\"128\"/>\n"
	out.print "	<Parameter Name=\"Index of Refraction\" Type=\"Real\" Value=\"#{ior}\"/>\n"
	out.print "	<Parameter Name=\"Specular Sampling\" Type=\"Boolean\" Value=\"0\"/>\n"
	out.print "	<Parameter Name=\"Transmitted Sampling\" Type=\"Boolean\" Value=\"0\"/>\n"
	out.print "	<Parameter Name=\"Specular Attenuation\" Type=\"String\" Value=\"Fresnel\"/>\n"
	out.print "	<Parameter Name=\"Transmitted Attenuation\" Type=\"String\" Value=\"Fresnel\"/>\n"
	out.puts "	</Object>"

	if mat.respond_to?(:name)
		if mat.display_name.upcase.include? "EMITFAKE"
			SU2KT.add_emitter(out,1.0,"0")
		end
		if mat.display_name.upcase.include? "EMIT["
			power=(mat.display_name.split('[')[1]).split(']')[0]
			SU2KT.add_emitter(out,power,"1")
		end
	end
end

#### Emitter description ####

def SU2KT::add_emitter(out,power,emit)
	out.puts "<Object Identifier=\"Diffuse Light\" Label=\"Diffuse Light\" Name=\"\" Type=\"Emittance\">"
	out.puts "	<Object Identifier=\"./Radiance/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "	<Parameter Name=\"Color\" Type=\"RGB\" Value=\"#{power} #{power} #{power}\"/>"
	out.puts "	</Object>"
	out.puts "	<Parameter Name=\"Attenuation\" Type=\"String\" Value=\"Inverse Square\"/>"
	out.puts "	<Parameter Name=\"Emitter\" Type=\"Boolean\" Value=\"" + emit + "\"/>"
	out.puts "	<Parameter Name=\"Front Side\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "	<Parameter Name=\"Back Side\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "</Object>"
end

##### --- EXPORT CLAY MATERIAL -----#####

def SU2KT::export_clay(out)
	out.puts "<Object Identifier=\"Whitted Material\" Label=\"Whitted Material\" Name=\"Clay\" Type=\"Material\">"
	out.puts "	<Object Identifier=\"./Ambient/Null Texture\" Label=\"Null Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "	</Object>"
	out.puts "	<Object Identifier=\"./Diffuse/Constant Texture\" Label=\"Constant Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "	<Parameter Name=\"Color\" Type=\"RGB\" Value=\"0.85 0.85 0.76\"/>"
	out.puts "	</Object>"
	out.puts "	<Parameter Name=\"Shininess\" Type=\"Real\" Value=\"128\"/>"
	out.puts "	<Parameter Name=\"Transmitted Shininess\" Type=\"Real\" Value=\"128\"/>"
	out.puts "	<Parameter Name=\"Index of Refraction\" Type=\"Real\" Value=\"3\"/>"
	out.puts "	<Parameter Name=\"Specular Sampling\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "	<Parameter Name=\"Transmitted Sampling\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "	<Parameter Name=\"Specular Attenuation\" Type=\"String\" Value=\"Fresnel\"/>"
	out.puts "	<Parameter Name=\"Transmitted Attenuation\" Type=\"String\" Value=\"Fresnel\"/>"
	out.puts "</Object>"
end

### Add Bump description ###

def SU2KT::add_bump(out,mat)

	out.puts "<Object Identifier=\"Bump Mapping\" Label=\"Bump Mapping\" Name=\"\" Type=\"Intersection Modifier\">"
	out.puts "<Object Identifier=\"./Texture/Weighted Texture\" Label=\"Weighted Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "<Object Identifier=\"Bitmap Texture\" Label=\"Bitmap Texture\" Name=\"\" Type=\"Texture\">"
	out.puts "<Parameter Name=\"Filename\" Type=\"String\" Value=\"#{$g_su2kt_model_textures[mat][4]}\"/>"
	projection=SU2KT.override_projection
	out.puts projection
	out.puts "<Parameter Name=\"Smooth\" Type=\"Boolean\" Value=\"1\"/>"
	out.puts "<Parameter Name=\"Inverted\" Type=\"Boolean\" Value=\"0\"/>"
	out.puts "</Object>"
	out.puts "<Parameter Name=\"Bitmap Texture:Weight\" Type=\"Real\" Value=\"1\"/>"
	out.puts "</Object>"
	out.puts "<Parameter Name=\"Strength\" Type=\"Real\" Value=\"1\"/>"
	out.puts "</Object>"

end

#### -- IMPORT KT MATERIAL FROM LIBRARY --- ###

def SU2KT::import_kt_material
	SU2KT.reset_global_variables
	materials=SU2KT.collect_kt_materials
	return if materials==nil
	selection=Sketchup.active_model.selection
	su_mats=SU2KT.selection_2_SU_mats(selection) if selection.length!=0
	return if selection.length!=0 and su_mats==nil
	kt_mats=SU2KT.select_material(materials)
	return if kt_mats==nil

	if Sketchup.active_model.materials.length!=0
		if selection.length==0
			number=SU2KT.assign_or_create(kt_mats)
		else
			number=SU2KT.assign_kt_mat(kt_mats,$su_materials)
			$su_materials=nil
			number=[0,number]
		end
	else
		number=SU2KT.add_materials(kt_mats,nil)
		number=[number,0]
	end

	text="#{number[0]} material added.\n#{number[1]} material(s) attached." 
	UI.messagebox (text)
end

def SU2KT::collect_kt_materials #Collect libraries and material names
	path=SU2KT.get_kt_path
	return if path==nil
	path=File.join(path.split($g_su2kt_ds))
	path=File.dirname(path)+"/MaterialEditor/Libraries/*/*.names"
	files=Dir[path]
	folders=[]
	files.each do |file|
		file_body=File.new(file,"r")
		content=[]
		while (line = file_body.gets)
			content.push line.chomp
		end
		file_body.close
		folders.push([File.basename(file,".names"),content])
	end
	folders
end

def SU2KT::select_material(folders) #Select materials to be imported
	folders.reverse!
	dialog_limit=12
	i=0
	prom=[]
	drop=[]
	val=[]
	folders.each do |lib|
		prom.push lib[0]
		drop.push (["-None-",lib[1],"-All-"].join("|"))
		val.push "-None-"
	end
	results=SU2KT.display_dialog(prom,drop,val,"Import KT Materials")
	return nil if results==nil
	path=SU2KT.get_kt_path
	path=File.join(path.split($g_su2kt_ds))
	path=File.dirname(path)+"/MaterialEditor/Libraries/"
	kt_mats=[]
	i=0
	folders.each do |folder|
			kt_mats.push([path+"#{folder[0]}/#{folder[0]}.xml",results[i]]) if folder[1].include? results[i]
			if results[i]=="-All-"
				folder[1].each do |mat|
				kt_mats.push([path+"#{folder[0]}/#{folder[0]}.xml",mat])
				end
			end
			i+=1
	end
	kt_mats
end

def SU2KT::assign_or_create(kt_mats)
	prom=[]
	drop=[]
	val=[]
	kt_mats.each do |kt_mat|
	prom.push kt_mat[1]
	drop.push ["Create new","Attach to existing  "].join("|")
	val.push "Create new"
	end
	results=SU2KT.display_dialog(prom,drop,val,"Create New or Attach to Existing SU Material")
	assign_me=[]
	create_me=[]
	(kt_mats.length).times do |i|
		if results[i]=="Create new"
			create_me.push kt_mats[i]
		else
			assign_me.push kt_mats[i]
		end
	end
		su_mats=[]
		model=Sketchup.active_model
		model.materials.each {|mat| su_mats.push mat.display_name}
		su_mats.sort!
	assign_me=SU2KT.assign_kt_mat(assign_me,su_mats) if assign_me!=[]
	SU2KT.add_materials(create_me,nil) if create_me!=[]
	return [create_me.length,assign_me]
end

def SU2KT::assign_kt_mat(kt_mats,su_mats)
	return nil if su_mats==[]
	model=Sketchup.active_model
	prom=[]
	drop=[]
	val=[]
	kt_mats.each do |kt_mat|
		prom.push kt_mat[1]
		drop.push((["-None-"]+su_mats).join("|"))
		val.push "-None-"
	end
	results=SU2KT.display_dialog(prom,drop,val,"Attach KT material to:")
	return if results==nil
	number=0
	model.start_operation("KT Materials attach")
	results.length.times do |i|
		if results[i]!="-None-" and results[i]!=nil
			SU2KT.add_materials([kt_mats[i]],model.materials[results[i]])
			number+=1
		end
	end
	model.commit_operation
	number
end

def SU2KT::add_materials(kt_mats,mat) #Attach KT materials to SU mats
	return 0 if kt_mats==[]
	model=Sketchup.active_model
	model.start_operation("Create KT Mat")
	update=(mat!=nil)
	materials=model.materials
	number=0
	kt_mats.each do |kt_mat|
		if SU2KT.check_library(kt_mat[0])!=nil

			mat=materials.add kt_mat[1] if update==false
			mat.set_attribute("su2kt","kt_lib_path",kt_mat[0])
			mat.set_attribute("su2kt","kt_name",kt_mat[1])

			kt_data=SU2KT.load_material(kt_mat[0],kt_mat[1]) #Get mat description from the library

			mat.set_attribute("su2kt","kt_mat",kt_data[0])
			mat.set_attribute("su2kt","kt_map",kt_data[1])

			mat.color=[255,0,174] if update==false #kt_data[2] if kt_data[2]!=[]
			if kt_data[3]!=[]
				default_size=60
				default_size=mat.texture.height if mat.texture!=nil
				tex=kt_data[3].rassoc "DIFF"
				if tex!=nil
					mat.texture=tex[0]
				else
					mat.texture=kt_data[3].first.first
				end
				mat.texture.size=default_size
			end
			number+=1
		end
	end
	model.commit_operation
	return number
end

def SU2KT::load_material(file,kt_mat) #Extracr material from the KT library with given path
	file_body=File.new(file,"r")
	first_line=SU2KT.find_mat_line(file_body,kt_mat)
	kt_data=SU2KT.extract_kt_material(file_body,first_line)
	file_body.close
	kt_data
end

def SU2KT::find_mat_line(file_body,kt_mat)
	found=false
	while found==false
		line = file_body.gets
		if line.include?"Label=\"Default Model\"" and line.include?"Type=\"Model\""
			line=file_body.gets
			found=true if (line[0..20+kt_mat.length]=="<Object Identifier=\"#{kt_mat}\"")
			found=true if kt_mat=="Sky Portal" and line.include?"Object Identifier=\"Sky Portal Light\""
		else
			line=nil
		end
	end
	return line
end

def SU2KT::extract_kt_material(file_body,first_line)

	kt_data=[]
	content_mat=[]
	content_mat.push first_line
	content_map=[]
	color=[]
	textures=[]
	map_param=false
	object=2 # Def Model & Mat

	while object!=0
		line = file_body.gets
		object+=1 if line[0..6]=="<Object"
		object-=1 if line[0..8]=="</Object>"

		tex_type=SU2KT.texture_type(line) if tex_type==nil

		if line.include? "Parameter Name=\"Filename\""
			line_split=line.split "\""
			file_name=File.basename(line_split[-2])
			file_name=File.dirname(file_body.path)+"/"+file_name
			line_split[-2]=file_name
			line=line_split.join("\"")
			textures.push [file_name,tex_type]
			tex_type=nil
		end
		if line.include? "Parameter Name=\"Projection\" Type=\"String\""
			5.times {file_body.gets}
			content_mat.push SU2KT.override_projection
			line=file_body.gets
		end
		if line.include? "Parameter Name=\"Map Channel\""
			file_body.gets #Omit also next line
			map_param=true
		else
			content_mat.push line if map_param==false and object!=0
			content_map.push line if map_param==true and object!=0
		end
	end

	return [content_mat.to_s,content_map.to_s,color,textures]
end

def SU2KT::texture_type(line)
	tex_type=nil
	tex_type="DIFF" if line.include? "Object Identifier=\"./Diffuse"
	tex_type="BUMP" if line.include? "Object Identifier=\"Bump Mapping"
	tex_type="SPEC" if line.include? "Object Identifier=\"./Specular"
	tex_type="REFL" if line.include? "Object Identifier=\"./Reflection"
	tex_type="REFR" if line.include? "Object Identifier=\"./Refraction"
	tex_type="LUMI" if line.include? "Object Identifier=\"./Radiance"
	return tex_type
end

def SU2KT::override_projection
	def_projection="<Parameter Name=\"Projection\" Type=\"String\" Value=\"UV\"/>\n<Parameter Name=\"Offset X\" Type=\"Real\" Value=\"0\"/>\n"
	def_projection+="<Parameter Name=\"Offset Y\" Type=\"Real\" Value=\"0\"/>\n<Parameter Name=\"Scale X\" Type=\"Real\" Value=\"1\"/>\n"
	def_projection+="<Parameter Name=\"Scale Y\" Type=\"Real\" Value=\"1\"/>\n<Parameter Name=\"Rotation\" Type=\"Real\" Value=\"0\"/>\n"
end

def SU2KT::check_library(file) #if exists
	return file if FileTest.exist? file
	path=SU2KT.get_kt_path
	return nil if path==nil
	path=File.join(path.split($g_su2kt_ds))
	path=File.dirname(path)+"/MaterialEditor/Libraries/"
	file=path+File.join(file.split("/")[-2..-1])
	return nil if not FileTest.exist? file
end

def SU2KT::selection_2_SU_mats(selection)
	SU2KT.status_bar("Collecting SU materials from Selection")
	$su_materials=[]
	SU2KT.collect_SU_materials(selection)
	SU2KT.status_bar("")
	SU2KT.status_bar("No material to attach KT material found.") if $su_materials==[] 
	return nil if $su_materials==[] 
	$su_materials.uniq! if $su_materials.length>1
	$su_materials.sort!
end

def SU2KT::collect_SU_materials(selection)
	for e in selection
		if e.class == Sketchup::Group and e.layer.visible? and e.visible?
			$su_materials.push e.material.display_name if e.material!=nil
			SU2KT.collect_SU_materials(e.entities)
		end
		if e.class == Sketchup::ComponentInstance and e.layer.visible? and e.visible?
			SU2KT.collect_SU_materials(e.definition.entities)
			$su_materials.push e.material.display_name if e.material!=nil
		end
		if e.class==Sketchup::Face
			$su_materials.push e.material.display_name if e.material!=nil
			$su_materials.push e.back_material.display_name if e.back_material!=nil#$su_materials.push e.back_material.display_name
		end
	end
end

### KT Materiels manager ###

def SU2KT::kt_mats_manager
	SU2KT.reset_global_variables
	mats=SU2KT.find_kt_mats
	if mats!=[]
		results=SU2KT.select_kt_dict(mats)
	else
		UI.messagebox "No KT materials imported\\attached yet."
	end
	SU2KT.edit_mats(mats,results) if results!=nil
end

def SU2KT::find_kt_mats
	mats=Sketchup.active_model.materials
	edit_pack=[]
	mats.each {|mat| edit_pack.push mat if (mat.attribute_dictionary "su2kt")}
	edit_pack
end

def SU2KT::select_kt_dict(mats)

	installed_mats=SU2KT.collect_kt_materials
	prom=[]
	drop=[]
	val=[]
	mats.each do |mat|
	dict=mat.attribute_dictionary "su2kt"
	prom.push mat.display_name
	libname=dict["kt_lib_path"].split("/")[-2]
	kt_mat=libname+" \\ #{dict["kt_name"]}" #Lib_name\kt_mat_name

	if SU2KT.is_installed(libname,dict["kt_name"],installed_mats)
		drop.push (["- Detach KT material", kt_mat ,"< Update"].join("|"))
	else
		drop.push (["- Detach KT material", kt_mat].join("|"))
	end
	val.push(kt_mat)
	end
	results=SU2KT.display_dialog(prom,drop,val,"KT Materials Update\\Detach")
end

def SU2KT.is_installed(lib_name,kt_name,installed)
	found=false
	installed.each do |ilib,names|
		names.each {|name| found=true if name==kt_name} if ilib==lib_name
	end
	return found
end


def SU2KT::edit_mats(mats,results)

	assign_me=[]
	updated=0
	mats.length.times do |i|
	mats[i].delete_attribute "su2kt" if results[i]=="- Detach KT material"
		if results[i]=="< Update"
			dict=mats[i].attribute_dictionary "su2kt"
			updated+=SU2KT.add_materials([[dict["kt_lib_path"],dict["kt_name"]]],mats[i])
		end
	end
	UI.messagebox updated.to_s+" material(s) updated." if updated!=0
end

def SU2KT::display_dialog(prompts,dropdowns,values,wind_name)
	dialog_limit=12
	i=0
	results=[]
	prom=[]
	drop=[]
	val=[]
	len=prompts.length.to_i
	while prompts.length>0
	prom.push prompts.shift
	drop.push dropdowns.shift
	val.push values.shift
	i+=1
		if (i/dialog_limit)!=((i-1)/dialog_limit) or i==len
			number=(len/dialog_limit.to_f).ceil
			res = inputbox(prom,val,drop,wind_name+" - #{(i/dialog_limit.to_f).ceil} of #{number}")
			return nil if res==false
			prom.clear
			drop.clear
			val.clear
			results+=res
		end
	end
	results
end

#### - Send text to status bar - ####
def SU2KT::status_bar(stat_text)
	statbar = Sketchup.set_status_text stat_text
end

### ------ Function RGB_2_HSB -------- ###
def SU2KT::rgb_to_hsb(r, g, b)

	r/=255.0
	g/=255.0
	b/=255.0

	rgb_max = [r,g,b].max
	rgb_min = [r,g,b].min

	v = rgb_max.to_f

	if rgb_max != 0
		s = (rgb_max - rgb_min).to_f / (rgb_max).to_f
	else
		s = 0.0
		h = 0.0
		return h,s,v*100.0
	end

	if r==g && g==b
		h=0
	else
		h = ( g-b )/ (rgb_max - rgb_min).to_f if r == rgb_max
		h = 2.0 + (b-r).to_f/(rgb_max - rgb_min).to_f if g == rgb_max
		h = 4.0 + (r-g).to_f/(rgb_max - rgb_min).to_f if b == rgb_max
	end

	h+=6 if h<0
	return ((h*600.0).round)/10.0,((s*1000.0).round)/10.0,((v*1000.0).round)/10.0
end

### ------ Function HSB_2_RGB -------- ###

def SU2KT::hsb_to_rgb(h,s,v)

	if ((s == 0) && (h == 0))
		r = g = b = v
	end
	v/=100.0
	s/=100.0
	h = 0.0 if (h == 360)

	h /= 60.0
	i = h.floor
	f = h - i
	p = v*(1 - s)
	q = v*(1 - s*f)
	t = v*(1 - s*(1 - f))

	case i
	when 0
		r = v
		g = t
		b = p
	when 1
		r = q
		g = v
		b = p
	when 2
		r = p
		g = v
		b = t
	when 3
		r = p
		g = q
		b = v
	when 4
		r = t
		g = p
		b = v
	when 5
		r = v
		g = p
		b = q
	end

	r*=255.0
	g*=255.0
	b*=255.0

	return r.round,g.round,b.round
end

##### -----  Animation export ------- ######

def SU2KT::animation

	SU2KT.reset_global_variables
	$animation=true
	$g_su2kt_textures_prefix = ".."+$g_su2kt_ds+ $g_su2kt_textures_prefix

	model = Sketchup.active_model
	pages = model.pages
	$frame=0

	$frame_per_sec = %w[30 25 23.98 20 15 10 5].join("|")
	loop_cam = %w[Yes No].join("|")
	$anim_sun = %w[Yes No].join("|")
	face_me = %w[Yes No].join("|")
	resolution = %w[Model-Inherited 320x240 640x480 768x576 800x600].join("|")

	dropdowns = [$frame_per_sec,loop_cam,$anim_sun,face_me,resolution]
	prompts=["Frames per second   ","Loop to first camera  ","Animated Lights and Sun?   ","Face-Me Components?  ","Resolution  "]
	values=SU2KT.get_stored_values
	results = inputbox prompts,values, dropdowns, "Animation export options"
	return nil if not results
	SU2KT.store_values(results)

	$frame_per_sec=results[0].to_f
	if results[1]=="Yes";loop_cam=true
		else ; loop_cam=false
	end
	if results[2]=="Yes";$anim_sun=true
		else ; $anim_sun=false
	end

	if results[3]=="Yes";face_me=true
		else ; face_me=false
	end

	if results[4]=="Model-Inherited"
		$resolution="4x4"
	else ; $resolution=results[4]
	end

	model_filename = File.basename(model.path)
	if model_filename!=""
		model_name = model_filename.split(".")[0]
		model_name += ".kst"
	else
		model_name = "Untitled_Anim.kst"
	end

	$g_su2kt_export_file=UI.savepanel("Export Animation Path", "" , model_name)

	return if $g_su2kt_export_file==nil

	$g_su2kt_model_name=File.basename($g_su2kt_export_file)
	$g_su2kt_model_name=model_name.split(".")[0]

	if $g_su2kt_export_file==$g_su2kt_export_file.split(".")[0]
		$g_su2kt_export_file+=".kst"
	end

	# $frames_path="Anim_"+File.basename($g_su2kt_export_file).split(".")[0]
	$frames_path=File.dirname($g_su2kt_export_file)+$g_su2kt_ds+"Anim_"+File.basename($g_su2kt_export_file).split(".")[0]
	
	if FileTest.exist? ($frames_path)
	else
			Dir.mkdir($frames_path)
	end

	time=pages.slideshow_time

	if loop_cam==true
		kt_page=pages.add "SU2KT_Loop"
		transit=[]
		pages.each {|page| transit.push(page.transition_time) if page.label==page.name}
		add_time=transit[0]
		pages.show_frame_at $frame
		kt_page.update
		kt_page.transition_time=add_time
		add_time = Sketchup.active_model.options["PageOptions"]["TransitionTime"] if add_time==-1	#default transition time
		time+=add_time
	end

	model_file=$g_su2kt_export_file.split(".")[0]+".xml"

	if time > 0

		SU2KT.set_merge_settings

		$out = File.new($g_su2kt_export_file,"w")

		$out.puts "message \"Load #{File.basename(model_file)}\""

		$hold_on=true
		if face_me==true
			Sketchup.active_model.active_view.animation = AnimCamera.new #Start animation of the camera
			while $hold_on==true
				choice=UI.messagebox "Exporting FaceMe components\nPlease wait and press OK when asked to\n-check status line at the bottom\n", MB_OKCANCEL
				if choice==2
					Sketchup.active_model.active_view.animation = nil
					$frame_file.close if !$frame_file.closed?
					$out.close
					pages.show_frame_at time
					Sketchup.send_action "pageDelete:" if loop_cam==true
					return
				end
			end

		else #No Face_me option
			while $frame/$frame_per_sec<time
				pages.show_frame_at $frame/$frame_per_sec

				frame_file=File.new("#{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".xml","w")
				SU2KT.export_global_settings(frame_file)
				SU2KT.write_sky(frame_file)
				SU2KT.param_cam_view(model.active_view, frame_file)
				SU2KT.export_lights(frame_file)

				SU2KT.finish_close(frame_file)

				$out.puts "message \"Merge '#{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".xml' #{$merge_settings}\""
				if $anim_sun==true
					SU2KT.generate_sun ($out)
				end
				$out.puts "message \"Render\""
				$out.puts "message \"SaveImage #{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".jpg\""

				SU2KT.lock_cache if $anim_sun==false and $frame==0

				$frame+=1
			end
		end

		$out.close
		pages.show_frame_at time+0.1
		Sketchup.send_action "pageDelete:" if loop_cam==true

		result=UI.messagebox "Animation path exported and saved into   \n#{$g_su2kt_export_file}\n\nLength: #{time} sec.\n\nRender exported animation in Kerkythea?",MB_YESNO

		SU2KT.render_animation if result==6
	else
		UI.messagebox "No pages present."
	end

	Sketchup.active_model.active_view.animation = nil

end

def SU2KT::lock_cache
	$out.puts "message \"./Irradiance Estimators/Density Estimation/Lock Cache\""
	$out.puts "message \"./Irradiance Estimators/Diffuse Interreflection/Lock Cache\""
end


def SU2KT::set_merge_settings
	$merge_settings="5 " #replace meshes keep materials
	if $anim_sun==true
		$merge_settings+="1 " #replace lights
	else
		$merge_settings+="0 " #keep lights
	end
	if $resolution=="4x4"
		$merge_settings+="5 " #keep camera settings modify possition
	else
		$merge_settings+="4 " #repalce camera & resolution
	end
	$merge_settings+="0 " #keep render settings
	if $anim_sun==true
		$merge_settings+="1" #replace global settings
	else
		$merge_settings+="0" #keep global settings
	end
end

def SU2KT::get_stored_values
	model=Sketchup.active_model
	dict_name="su2kt"
	if model.attribute_dictionary dict_name
		anim_fps=model.get_attribute(dict_name, "anim_fps")
		anim_loop=model.get_attribute(dict_name, "anim_loop")
		anim_animsun=model.get_attribute(dict_name, "anim_animsun")
		anim_faceme=model.get_attribute(dict_name, "anim_faceme")
		anim_resol=model.get_attribute(dict_name, "anim_resol")
		values=[anim_fps,anim_loop,anim_animsun,anim_faceme,anim_resol]
	else
		model.attribute_dictionary(dict_name, true)
		values=["15","No","No","No","640x480"]
		SU2KT.store_values values
	end
	return values
end

def SU2KT::store_values (values)
	model=Sketchup.active_model
	dict_name="su2kt"
	model.set_attribute(dict_name,"anim_fps",values[0])
	model.set_attribute(dict_name,"anim_loop",values[1])
	model.set_attribute(dict_name,"anim_animsun",values[2])
	model.set_attribute(dict_name,"anim_faceme",values[3])
	model.set_attribute(dict_name,"anim_resol",values[4])
end

##### ----- Frames renumbering ----- ####

def SU2KT::add_zeros(frame)

	total_frames=(Sketchup.active_model.pages.slideshow_time/(1.0/$frame_per_sec)).to_i
	str_length=total_frames.to_s.length
	frame_length=frame.to_s.length
	name="0"*(str_length+2)
	name[str_length+1-frame_length..str_length+1]=frame.to_s
	return name

end

##### -----  FaceMe Components export ------- ######

def SU2KT::export_faceme

	SU2KT.reset_global_variables

	model = Sketchup.active_model
	pages = model.pages

	model_filename = File.basename(model.path)
	if model_filename!=""
		model_name = "FM_"+model_filename.split(".")[0]
		model_name += ".xml"
	else
		model_name = "Untitled_FaceMe.xml"
	end

	$g_su2kt_export_file=UI.savepanel("Export FaceMe Components & Sun", "" , model_name)

	return if $g_su2kt_export_file==nil

	$g_su2kt_model_name=File.basename($g_su2kt_export_file)
	$g_su2kt_model_name=model_name.split(".")[0]

	if $g_su2kt_export_file==$g_su2kt_export_file.split(".")[0]
		$g_su2kt_export_file+=".xml"
	end

	out = File.new($g_su2kt_export_file,"w")

		SU2KT.export_global_settings(out)

		SU2KT.write_sky(out)
		SU2KT.param_cam_view(model.active_view, out)
		SU2KT.write_sun(out)

		SU2KT.collect_entities(Sketchup.active_model.entities, Geom::Transformation.new)
		SU2KT.export_fm_faces(out)

		SU2KT.finish_close(out)

	UI.messagebox "Face Me Components saved into   \n\n#{$g_su2kt_export_file}\n"

end

# Start insert light tool
def SU2KT::instert_point
	Sketchup.active_model.select_tool SU2KTL.new("su2kt_pointlight.skp")
end

def SU2KT::insert_spot
	Sketchup.active_model.select_tool SU2KTL.new("su2kt_spotlight.skp")
end

#Validation of selection

	def SU2KT::point_selected
		s = Sketchup.active_model.selection
		model=Sketchup.active_model
		ss=model.selection.first
		cName=ss.definition.name if not s.empty? and ss.class == Sketchup::ComponentInstance

		return false if ((s.empty?) || (s.length > 1))
		return ss if (s.first.class == Sketchup::ComponentInstance and cName.include? "su2" and cName.include? "_pointlight")
	end

	def SU2KT::spot_selected
		s = Sketchup.active_model.selection
		model=Sketchup.active_model
		ss=model.selection.first
		cName=ss.definition.name if not s.empty? and ss.class == Sketchup::ComponentInstance

		return false if ((s.empty?) || (s.length > 1))
		return ss if (s.first.class == Sketchup::ComponentInstance and cName.include? "su2" and cName.include? "_spotlight")
	end

end #Class end

class AnimCamera

def initialize
	$frame=0
	@slide_time=0.0
end

def nextFrame(view)
	Sketchup::set_status_text($frame, 2)

	@slide_time=$frame/$frame_per_sec

	Sketchup.active_model.pages.show_frame_at @slide_time

	$frame_file=File.new("#{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".xml","w")
		SU2KT.export_global_settings($frame_file)
		SU2KT.write_sky($frame_file)
		SU2KT.param_cam_view(Sketchup.active_model.active_view, $frame_file)
		SU2KT.export_lights($frame_file)
		#SU2KT.write_sun($frame_file)

		SU2KT.collect_entities(Sketchup.active_model.entities, Geom::Transformation.new)
		SU2KT.export_fm_faces($frame_file)
		SU2KT.finish_close($frame_file)

		$out.puts "message \"Merge '#{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".xml' #{$merge_settings}\""
		if $anim_sun==true
			SU2KT.generate_sun ($out)
		end
		$out.puts "message \"Render\""
		$out.puts "message \"SaveImage #{$frames_path}"+$g_su2kt_ds+SU2KT.add_zeros($frame)+".jpg\""

	SU2KT.lock_cache if $anim_sun==false and $frame==0

	$frame+=1
	if $frame/$frame_per_sec >= Sketchup.active_model.pages.slideshow_time
		self.stop
		$hold_on=false
		SU2KT.status_bar("Exporting Face Me Components FINISHED. Press OK!")
		return false
	end
	return true
end

def stop
	Sketchup::set_status_text("", 1)
	Sketchup::set_status_text("", 2)
end

end # class AnimCamera

#--------------------------------------- Lights placing tool ----------------------
class SU2KTL

def initialize(comp)
	@ip1 = nil
	@ip1 = nil
	@drawn=false
	@compname=comp
	model=Sketchup.active_model
	status=model.start_operation("Insert Light")
	model.commit_operation
end

def activate
	@ip = Sketchup::InputPoint.new
	@ip1 = Sketchup::InputPoint.new
	@ip2 = Sketchup::InputPoint.new
	SU2KT::status_bar("Select instertion point of the light")
	self.reset(nil)
end

def deactivate(view)
	view.invalidate
end

def onMouseMove(flags, x, y, view)
	if @state==0
		@ip.pick view, x, y
		if( @ip != @ip1 )
			view.invalidate if( @ip.display? or @ip1.display? )
			@ip1.copy! @ip
			view.tooltip = @ip1.tooltip if( @ip1.valid? )
		end
	else
		@ip2.pick view, x, y, @ip1
		view.tooltip = @ip2.tooltip if( @ip2.valid? )
		view.invalidate
	end
end

def onLButtonDown(flags, x, y, view)
	if (@state==0)
		@ip1.pick view, x, y
		if( @ip1.valid? )
			Sketchup::set_status_text "Select pointlight\'s range", SB_PROMPT if (@compname=="su2kt_pointlight.skp")
			Sketchup::set_status_text "Select spotlight\'s target", SB_PROMPT if (@compname=="su2kt_spotlight.skp")
			@state=1
		end
	elsif (@state==1)
		if(@ip2.valid?)
			self.create_light(view)
			self.reset(view)
			@state=0
		end
	end
end

def draw(view)

	if( @ip1.valid? )
		if( @ip1.display? )
			@ip1.draw(view)
			@drawn = true
		end
	end
	if( @ip2.valid? )
		@ip2.draw(view) if( @ip2.display?)
		length = (@ip1.position.distance(@ip2.position)).to_m
		self.draw_geometry(@ip1.position, @ip2.position, view)
		@drawn = true
	end
end

def onCancel(flag, view)
	mod=Sketchup.active_model
	mod.select_tool nil if @state==0
	self.reset(view)
end

def reset(view)
	@state = 0
	SU2KT::status_bar("Select instertion point of the light")
	@ip.clear
	@ip1.clear
	@ip2.clear
	if( view )
		view.tooltip = nil
		view.invalidate
	end
	@drawn = false
end

def draw_geometry(pt1,pt2,view)
	view.set_color_from_line(@ip1, @ip2)
	view.line_stipple="-.-"
	view.line_width = 1
	view.draw(GL_LINE_STRIP, pt1, pt2)
end

def deactivate(view)
	view.invalidate if @drawn
end

def create_light(view)
	pt1=@ip1.position
	pt2=@ip2.position
	model=Sketchup.active_model
	definitions=model.definitions
	path=Dir[$:.first.split('/')[0..-2].join('/')+"/Components/**/#{@compname}"].first
	return if path==nil #TO DO - create light if no skp
	definition=definitions.load(path.to_s)
	model.start_operation("Insert Light")
	light=model.active_entities.add_instance(definition, @ip1.position)
	vector=pt1 - pt2
	vector2=vector.clone
	vector2.length=0.1 #2.5 mm
	pt1=pt1-vector2
	vector2=vector
	vector2=Geom::Vector3d.new(0,0,1) if @compname=="su2kt_pointlight.skp"
	trans=Geom::Transformation.new(pt1,vector2)
	light.transformation=trans
	dist=vector.length.to_m
	SU2KT.set_pointlight_params(light,dist) if @compname=="su2kt_pointlight.skp"
	SU2KT.set_spotlight_params(light,dist) if @compname=="su2kt_spotlight.skp"
	model.commit_operation

end

end # class SU2KT_Light insert

# ---- Menu items and icons ---- #
if( not file_loaded?("su2kt.rb") )

	main_menu = UI.menu("Plugins").add_submenu("Kerkythea Exporter")
	main_menu.add_item("Export Model") {SU2KT.export}
	main_menu.add_separator
	main_menu.add_item("Export Animation Path") {SU2KT.animation}
	main_menu.add_separator
	main_menu.add_item("Export Face Me Components and Sun") {SU2KT.export_faceme}
	main_menu.add_separator
	main_menu.add_item("Import KT materials") {SU2KT.import_kt_material}
	main_menu.add_separator
	main_menu.add_item("KT Materials Update\\Detach") {SU2KT.kt_mats_manager}
	main_menu.add_separator
	main_menu.add_item("About SU2KT") {SU2KT.about}

	UI.add_context_menu_handler do |menu|

	menu.add_separator if ( SU2KT.point_selected ) or (SU2KT.spot_selected)
	menu.add_item("SU2KT: Edit Pointlight") { SU2KT.set_pointlight_params(SU2KT.point_selected,nil) } if SU2KT.point_selected
	menu.add_item("SU2KT: Edit Spotlight") { SU2KT.set_spotlight_params(SU2KT.spot_selected,nil) } if SU2KT.spot_selected

	end

	dir= File.dirname(__FILE__)
	cmd1 = UI::Command.new("Export Model") {SU2KT.export}
	cmd1.large_icon = cmd1.small_icon = dir+"/su2kt/kt_icon.png"
	cmd1.status_bar_text = cmd1.tooltip = "Export model to Kerkythea"

	cmd2 = UI::Command.new("Export Animation") {SU2KT.animation}
	cmd2.large_icon = cmd2.small_icon = dir+"/su2kt/kt_anim.png"
	cmd2.status_bar_text = cmd2.tooltip = "Export animation path to Kerkythea"

	cmd3 = UI::Command.new("Export FM Components and Sun") {SU2KT.export_faceme}
	cmd3.large_icon = cmd3.small_icon = dir+"/su2kt/kt_faceme.png"
	cmd3.status_bar_text = cmd3.tooltip = "Export Face Me Components and Sun"

	cmd4 = UI::Command.new("Insert Point Light") {SU2KT.instert_point}
	cmd4.large_icon = cmd4.small_icon = dir+"/su2kt/kt_point.png"
	cmd4.status_bar_text = cmd4.tooltip = "Insert Point Light"

	cmd5 = UI::Command.new("Insert Spot Light") {SU2KT.insert_spot}
	cmd5.large_icon = cmd5.small_icon = dir+"/su2kt/kt_spot.png"
	cmd5.status_bar_text = cmd5.tooltip = "Insert Spot Light"

	cmd6 = UI::Command.new("Import KT materials") {SU2KT.import_kt_material}
	cmd6.large_icon = cmd6.small_icon = dir+"/su2kt/kt_mats.png"
	cmd6.status_bar_text = cmd6.tooltip = "Import KT materials"

	cmd7 = UI::Command.new("Open KT Materials Manager") {SU2KT.kt_mats_manager}
	cmd7.large_icon = cmd7.small_icon = dir+"/su2kt/kt_manager.png"
	cmd7.status_bar_text = cmd7.tooltip = "KT Materials Update\\Detach"

	tb = UI::Toolbar.new("SU2Kerkythea")
	tb.add_item cmd1
	tb.add_item cmd2
	tb.add_item cmd3
	tb.add_item cmd4
	tb.add_item cmd5
	tb.add_item cmd6
	tb.add_item cmd7

	tb.show if tb.get_last_state == -1

end
file_loaded("su2kt.rb")